@@ -17,12 +17,13 @@ pub mod once;
 pub mod instance_of;
 pub mod reference_type;
 pub mod promise;
+pub mod discard;
 
 #[cfg(feature = "futures")]
 pub mod promise_future;
 
 #[cfg(feature = "futures")]
-pub mod promise_executor;
+pub mod executor;
 
 #[cfg(feature = "nightly")]
 pub mod void {
